import os

try:
    from setuptools import setup, find_packages
except ImportError:
    from distutils.core import setup, find_packages


PATH_TO_FILE = os.path.dirname(__file__)


with open(os.path.join(PATH_TO_FILE, 'README')) as f:
    long_description = f.read()


VERSION = (0, 1, 0)


# Dynamically calculate the version based on VERSION tuple
if len(VERSION) > 2 and VERSION[2] is not None:
    str_version = "%s.%s_%s" % VERSION[:3]
else:
    str_version = "%s.%s" % VERSION[:2]


version = str_version


setup(
    name='nose-cprof',
    version=version,
    description="A python nose plugin to profile using cProfile rather than the default Hotshot profiler.",
    long_description=long_description,
    author='Marc Sherry',
    author_email='msherry@gmail.com',
    url='https://github.com/msherry/nose-cprof',
    install_requires=[
        'nose >=1.0.0',
    ],
    setup_requires=[],
    test_suite='nose.collector',
    zip_safe=False,
    packages=find_packages(),
    classifiers=[
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Programming Language :: Python',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ],
    entry_points={
        'nose.plugins.0.10': [
            'nose_cprofile = nose_cprofile:Profile',
        ]
    },
)



import cProfile
import logging
import os

from nose.plugins.base import Plugin
from nose.util import tolist

log = logging.getLogger('nose.plugins')

class Profile(Plugin):
    """
    Use this plugin to run tests using the cProfile profiler.
    """

    # attributes required by nose
    enabled = True
    name = 'cprofile'

    pfile_name = None

    def options(self, parser, env):
        """Register commandline options.
        """
        Plugin.options(self, parser, env)
        parser.add_option('--cprofile-stats-file', action='store',
                          dest='profile_stats_file',
                          metavar="FILE",
                          default=env.get('NOSE_PROFILE_STATS_FILE'),
                          help='Profiler stats file; default "stats.dat"')
        parser.add_option("--cprofile-stats-erase", action="store_true",
                          default=env.get('NOSE_PROFILE_STATS_ERASE'),
                          dest="stats_erase",
                          help="Erase previously-collected profiling "
                          "statistics before run")

    def begin(self):
        """Instantiate profiler
        """
        self.prof = cProfile.Profile()

    def configure(self, options, conf):
        """Configure plugin.
        """
        Plugin.configure(self, options, conf)
        self.conf = conf
        if options.profile_stats_file:
            self.pfile_name = options.profile_stats_file
        else:
            self.pfile_name = 'stats.dat'
        self.fileno = None
        if options.stats_erase:
            self._erase_stats_file()

    def prepareTest(self, test):
        """Wrap entire test run in :func:`prof.runcall`.
        """
        log.debug('preparing test %s' % test)
        def run_and_profile(result, prof=self.prof, test=test):
            prof.runcall(test, result)
            prof.dump_stats(self.pfile_name)
        return run_and_profile

    def finalize(self, result):
        pass

    def _erase_stats_file(self):
        if os.path.exists(self.pfile_name):
            os.unlink(self.pfile_name)


if __name__ == '__main__':
    nose.main(addplugins=[Profile()])
